"""Pure python bindings for the netcf library

netcf is a cross-platform network configuration library.

It takes the description of a network interface in its own
platform-independent format and adapts the local system's network
configuration to provide that interface.

"""

#
# Copyright (C) 2010 Red Hat Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Lars Sjostrom <lars.sjostrom@redhat.com>

__author__ = "Lars Sjostrom <lars.sjostrom@redhat.com>"

import ctypes
import ctypes.util

def _dlopen(*args):
    """Search for one of the libraries given as arguments and load it.
    Returns the library.
    """
    libs = [l for l in [ ctypes.util.find_library(a) for a in args ] if l]
    lib  = reduce(lambda x, y: x or ctypes.cdll.LoadLibrary(y), libs, None)
    if not lib:
        raise ImportError("Unable to import lib%s!" % args[0])
    return lib

class Netcf(object):
    "Class wrapper for the netcf library"
    
    # Load libnetcf
    _libnetcf = _dlopen("netcf")
    _libnetcf.ncf_if_mac_string.restype = ctypes.c_char_p
    _libnetcf.ncf_if_name.restype = ctypes.c_char_p
    _libnetcf.ncf_if_xml_desc.restype = ctypes.c_char_p
    _libnetcf.ncf_if_xml_state.restype = ctypes.c_char_p

    # Netcf Flags
    NONE = 0
    NETCF_IFACE_INACTIVE = 1
    NETCF_IFACE_ACTIVE = 2
    
    def __init__(self, root=None):
        """Initialize the library.

        Use 'root' as the filesystem root. If 'root' is None it will use "/".
        
        """

        # Sanity checks
        if not isinstance(root, basestring) and root != None:
            raise TypeError("root MUST be a string or None!")

        class netcf(ctypes.Structure):
            pass

        class netcf_if(ctypes.Structure):
            pass

        # struct netcf *ncf
        self.ncf = ctypes.pointer(netcf())

        # struct netcf_if *nif
        self.nif = ctypes.pointer(netcf_if())

        # Create the Netcf object
        ret = Netcf._libnetcf.ncf_init(ctypes.byref(self.ncf), root)
        if ret != 0:
            raise RuntimeError("Unable to initiate netcf object!")


    def numOfInterfaces(self, flags=NETCF_IFACE_ACTIVE):
        """Returns number of known interfaces
        
        flags:
        Netcf.NETCF_IFACE_ACTIVE for active interfaces
        Netcf.NETCF_IFACE_INACTIVE for inactive interfaces
        
        """        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")
        if not isinstance(flags, int):
            raise Exception('flags MUST be an integer')

        # Call the function
        ret = Netcf._libnetcf.ncf_num_of_interfaces(self.ncf, flags)
        if ret < 0:
            raise Exception(self.__errorMsg())
        return ret

    def listInterfaces(self, flags=NETCF_IFACE_ACTIVE):
        """Returns an array of dictionaries of known interfaces and its 
        mac addresses
        
        flags:
        Netcf.NETCF_IFACE_ACTIVE for active interfaces
        Netcf.NETCF_IFACE_INACTIVE for inactive interfaces
        
        """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")
        if not isinstance(flags, int):
            raise Exception('flags MUST be an integer')
        
        # char **intf
        intf = ctypes.pointer(ctypes.c_char_p())

        devices = []
        numif = self.numOfInterfaces(flags)        
        nint = Netcf._libnetcf.ncf_list_interfaces(self.ncf, numif, intf, flags)
        if nint < 0:
            raise Exception(self.__errorMsg())

        for i in range(numif):
            devices.append(intf[i])
            intf[i] = None
        
        #clear array
        intf = None
        
        return devices
 
    def __lookupByName(self, name):
        """ Look interface up by name """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(name, basestring):
            raise TypeError("name MUST be a string!")     
        
        self.nif = Netcf._libnetcf.ncf_lookup_by_name(self.ncf, name)

    def define(self, xml):
        """ Define a new interface
        
        Returns the name of the newly created interface
        
        """        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(xml, basestring):
            raise TypeError("xml MUST be a string!")  
 
        self.nif = Netcf._libnetcf.ncf_define(self.ncf, xml)
        if self.nif == self.NONE:
            Netcf._libnetcf.ncf_if_free(self.nif)
            raise Exception(self.__errorMsg())

        return Netcf._libnetcf.ncf_if_name(self.nif)

    def undefine(self, ifname):
        """ Delete the interface definition """        
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )

        ret = Netcf._libnetcf.ncf_if_undefine(self.nif)
        if ret < 0:
            Netcf._libnetcf.ncf_if_free(self.nif)
            raise Exception(self.__errorMsg())

        return ret

    def ifup(self, ifname):
        """ Bring up interface """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )
        
        return Netcf._libnetcf.ncf_if_up(self.nif)
    
    def ifdown(self, ifname):
        """ Take down interface """
                
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )
        
        return Netcf._libnetcf.ncf_if_down(self.nif)
    
    def ifMacString(self, ifname):
        """ Return MAC address for interface """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )
        
        mac = Netcf._libnetcf.ncf_if_mac_string(self.nif)
        Netcf._libnetcf.ncf_if_free(self.nif)
        
        return mac
        
    def ifXMLDesc(self, ifname):
        """Produce an XML description for the static
        (stored) interface config
        """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )
        
        xml = Netcf._libnetcf.ncf_if_xml_desc(self.nif)
        if xml == self.NONE:
            Netcf._libnetcf.ncf_if_free(self.nif)
            raise Exception(self.__errorMsg())

        return xml

    def ifXMLState(self, ifname):
        """ Produce an XML description of the current live state of the
        interface, but potentially with extra info not contained in the 
        static config (ie the current IP address of an interface that uses DHCP)
        
        """
        
        # Sanity checks
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")        
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("Interface %s not found!" % ifname )
        
        xml = Netcf._libnetcf.ncf_if_xml_state(self.nif)
        if xml == self.NONE:
            Netcf._libnetcf.ncf_if_free(self.nif)
            raise Exception(self.__errorMsg())

        return xml
    
    def ifStatus(self, ifname):
        """  Report various status info about the interface
        
        return Netcf.NETCF_IFACE_INACTIVE or Netcf.NETCF_IFACE_ACTIVE
        """
        
        if not self.ncf:
            raise RuntimeError("The Netcf objevt has alreadt been closed!")
        if not isinstance(ifname, basestring):
            raise TypeError("ifname MUST be a string!")
        
        self.__lookupByName(ifname)
        if self.nif == self.NONE:
            raise Exception("interface %s not found!" % ifname)
        
        flags = ctypes.c_int()
        ret = Netcf._libnetcf.ncf_if_status(self.nif, ctypes.byref(flags))
        if ret != 0:
            Netcf._libnetcf.ncf_if_free(self.nif)
            raise Exception(self.__errorMsg())
        
        return flags.value
        
    def close(self):
        """ Close the Netcf object """        
        
        if not self.ncf:
            return
        
        Netcf._libnetcf.ncf_close(self.ncf)
        self.ncf = None

    def __errorMsg(self):
        """ Return the error code when a previous call failed """
        
        if not self.ncf:
            raise RuntimeError("The Netcf object has already been closed!")
    
        errmsg = ctypes.c_char_p()
        details = ctypes.c_char_p()

        errcode = Netcf._libnetcf.ncf_error(self.ncf, ctypes.byref(errmsg),
                                            ctypes.byref(details))
        if errcode != 0:
            return "%s (%s)" % (errmsg.value, details.value)
        
